@@ -6,11 +6,12 @@ from .simplemath import apply_offset, take_derivative1d
 import warnings
 import numpy as np
 from .parser import math_stream
+from scipy.interpolate import interp1d
 
 
-def loadSCAscans(basedir, file, x_stream, y_stream, *args, norm=True, is_XAS=False, xoffset=None, xcoffset=None, yoffset=None, ycoffset=None, background=None, deriv=None,energyloss=None):
+def loadSCAscans(basedir, file, x_stream, y_stream, *args, norm=True, is_XAS=False, xoffset=None, xcoffset=None, yoffset=None, ycoffset=None, background=None, deriv=None, energyloss=None, grid_x=[None, None, None]):
     special_streams = ['TEY', 'TFY', 'PFY', 'iPFY', 'XES', 'rXES', 'specPFY',
-                       'XRF', 'rXRF', 'XEOL', 'rXEOL', 'POY', 'TOY', 'EY']  # all special inputs
+                       'XRF', 'rXRF', 'XEOL', 'rXEOL', 'POY', 'TOY', 'EY', 'Sample', 'Mesh']  # all special inputs
     XAS_streams = ['TEY', 'TFY', 'PFY', 'iPFY', 'specPFY', 'POY',
                    'TOY', 'rXES', 'rXRF', 'rXEOL']  # All that are normalized to mesh
 
@@ -102,11 +103,33 @@ def loadSCAscans(basedir, file, x_stream, y_stream, *args, norm=True, is_XAS=Fal
             elif y_stream == 'EY':
                 return data[arg].sample_current
 
+            #elif y_stream == 'tey':
+            #    # spec mnemonic for sample current
+            #    return data[arg].sample_current
+
+            elif y_stream == 'Sample':
+                # to ensure backwards compatibility
+                return data[arg].sample_current
+
+            elif y_stream == 'Mesh':
+                # to ensure backwards compatibility
+                return data[arg].mesh_current
+
             else:
-                raise UserWarning("Special Stream not defined.")
+                try:
+                    return get_sca_data(y_stream, data, arg)
+
+                except:
+                    raise UserWarning("Special Stream not defined.")
 
         else:
-            return np.array(data[arg].sca_data[y_stream])
+            return get_sca_data(y_stream, data, arg)
+
+    def get_sca_data(stream, data, arg):
+        try:
+            return np.array(data[arg].sca_data[stream])
+        except:
+            return np.array(data[arg].sca_data[data[arg].mnemonic2name[stream]])
 
     def get_x_data(x_stream, data, arg, background, REIXSObj):
         if x_stream == "Mono Energy":
@@ -125,7 +148,18 @@ def loadSCAscans(basedir, file, x_stream, y_stream, *args, norm=True, is_XAS=Fal
             return np.array(range(0, len(data[arg].y_stream)), dtype=int)
 
         else:
-            return np.array(data[arg].sca_data[x_stream])
+            return get_sca_data(x_stream, data, arg)
+
+    def grid_data(x_stream, y_stream, grid):
+        xmin = grid[0]
+        xmax = grid[1]
+
+        numPoints = int(np.ceil((xmax-xmin)/grid[2])) + 1
+        new_x = np.linspace(xmin, xmax, numPoints)
+        f = interp1d(x_stream, y_stream, fill_value='extrapolate')
+        new_y = f(new_x)
+
+        return new_x, new_y
 
     data = dict()
     REIXSobj = REIXS(basedir, file)
@@ -138,6 +172,16 @@ def loadSCAscans(basedir, file, x_stream, y_stream, *args, norm=True, is_XAS=Fal
 
         # Assign the calculated result to the x_stream of the object in data dict
         data[arg].x_stream = math_stream(x_stream, data, arg, get_x_data)
+
+        # Grid the data if specified
+        if grid_x != [None, None, None]:
+            new_x, new_y = grid_data(
+                data[arg].x_stream, data[arg].y_stream, grid_x)
+
+            data[arg].x_stream = new_x
+            data[arg].y_stream = new_y
+
+        # Apply offsets and normalize
         data[arg].x_stream = apply_offset(
             data[arg].x_stream, xoffset, xcoffset)
 
@@ -148,6 +192,7 @@ def loadSCAscans(basedir, file, x_stream, y_stream, *args, norm=True, is_XAS=Fal
         data[arg].y_stream = apply_offset(
             data[arg].y_stream, yoffset, ycoffset)
 
+        # Take derivatives
         if deriv != None:
             data[arg].y_stream = take_derivative1d(
                 data[arg].y_stream, data[arg].x_stream, deriv)
@@ -155,7 +200,8 @@ def loadSCAscans(basedir, file, x_stream, y_stream, *args, norm=True, is_XAS=Fal
                 data[arg].y_stream = data[arg].y_stream / \
                     data[arg].y_stream.max()
 
-        if energyloss!=None:
+        # Transforms RIXS to energy loss scale if incident energy is given
+        if energyloss != None:
             data[arg].x_stream = energyloss-data[arg].x_stream
 
     return data
